@@ -23,6 +23,7 @@ The other feature is simply to
 
 __all__ = ['pool', 'pmap']
 
+import itertools as it
 # from aljunberg:  https://gist.github.com/aljungberg/626518
 from multiprocessing.pool import IMapIterator
 def wrapper(func):
@@ -49,6 +50,9 @@ def pool(n=None, dummy=True):
         from multiprocessing.dummy import Pool
     else:
         from multiprocessing import Pool
+    if n is None:
+        import multiprocessing
+        n = multiprocessing.cpu_count() - 1
 
     return Pool(n)
 
@@ -80,19 +84,29 @@ def pmap(f, iterable, n=None, dummy=False, p=None):
       p: existing pool to re-use
     """
 
+    # make it easier to debug.
+    if n == 1:
+        for r in it.starmap(f, iterable):
+            yield r
+        raise StopIteration
+
+
     if p is None:
         po = pool(n, dummy)
     else:
         po = p
     assert hasattr(po, 'imap')
     f = _func_star(f)
-    for r in po.imap(f, iterable):
-        yield r
+
+    try:
+        for r in po.imap(f, iterable):
+            yield r
 
     # explicitly clean up created pool
-    if p is None:
-        try:
-            po.terminate()
-            po.close()
-        except:
-            pass
+    finally:
+        if p is None:
+            try:
+                po.terminate()
+                po.close()
+            except:
+                pass
